<?php
/*
 #########################################################################
 #                       xt:Commerce Shopsoftware
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
 # Copyright 2021 xt:Commerce GmbH All Rights Reserved.
 # This file may not be redistributed in whole or significant part.
 # Content of this file is Protected By International Copyright Laws.
 #
 # ~~~~~~ xt:Commerce Shopsoftware IS NOT FREE SOFTWARE ~~~~~~~
 #
 # https://www.xt-commerce.com
 #
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
 # @copyright xt:Commerce GmbH, www.xt-commerce.com
 #
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
 # xt:Commerce GmbH, Maximilianstrasse 9, 6020 Innsbruck
 #
 # office@xt-commerce.com
 #
 #########################################################################
 */

set_time_limit ( 600 );
include_once _SRV_WEBROOT.'xtWizard/config.php';

include_once _SRV_WEBROOT._SRV_WEB_FRAMEWORK.'classes/class.plugin.php';
include_once _SRV_WEBROOT._SRV_WEB_FRAMEWORK.'classes/class.FileHandler.php';
include_once _SRV_WEBROOT._SRV_WEB_FRAMEWORK.'classes/class.ImageTypes.php';


class Update65xTo66x extends ExecutableScript {

    /**
     * (non-PHPdoc)
     * @see IExecutableScript::getWizardType()
     */
    public function getWizardType() {
        return Wizard::WIZARD_TYPE_UPDATE;
    }
    
    /**
     * Needed only for update scripts. Needed to determine if the script can be applied
     * to the current shop version
    */
    
    public function getAppliableShopVersion() {
        return '6.5.0';
    }


    public function getTargetShopVersion() {
        return '6.6.0';
    }
    public function getSkippableShopVersion(){
        return '6.6.0';
    }
    
    /**
     * Get pages
     * @return string
     */
    protected function getPagesXml() {
        return file_get_contents(dirname(__FILE__) . DS . 'pages.xml');
    }
    
    /**
     * Index action - testing quesries
     * @param WizardPage $page
     */
    public function displayStartPage(WizardPage $page) {
        $this->InitialSQL($page);
    }


    /**
     * Complete action
     * @param WizardPage $page
     */
    public function displayFinishPage(WizardPage $page)
    {
		$url = Wizard::getInstance()->buildUrl(array(
			Wizard::PARAM_SCRIPT => 'StartPageScript',
            Wizard::PARAM_PAGE => 'update',
        ));
        // Redirect to update page
        Wizard::getInstance()->redirect($url);
    }
    
    public function displayFinishInstructionsPage(WizardPage $page)
    {
        $page
            ->setTemplate('templates/info_page.html')
            ->assignTemplateVar('message', TEXT_FINISH_INSTRUCTIONS_63.
                '<br /><p class="alert alert-info">'.TEXT_COPY_VERSION_INFO.'</p>'.
                '<p class="alert">'.TEXT_INFO_PLACE_LICENSE.'</p>')
            ->assignTemplateVar('title', 'INFO')
            ->assignTemplateVar('next', true);
    }
    

    protected static function _getFileContent($filename) {
        $handle = fopen($filename, 'rb');
        $content = fread($handle, filesize($filename));
        fclose($handle);
        return $content;
    }

    public function _installSQL( $idb, $filename, $prefix='', $language_code='',$store_id='', $additionalSqlLines = false) {
        $query = '';
        // open sql
        $filename = dirname(__FILE__) . DS .$filename;
        $sql_content = $this->_getFileContent($filename);
        if($additionalSqlLines)
        {
            $sql_content .= PHP_EOL.$additionalSqlLines.PHP_EOL.' ';
        }
        $sql_content .= PHP_EOL.' ';

        // replace windows linefeeds
        $sql_content = str_replace("\r\n","\n",$sql_content);
        $queries = array();
        $chars = strlen($sql_content);
        for ($i = 0; $i < $chars; $i++) {
            // check if char is ; and next \n
            if ($sql_content[$i]==';' && $sql_content[$i+1]=="\n") {
                $query .= $sql_content[$i];
                $queries[] = $query;
                $query = '';
                $i++;
            } else {
                if ($sql_content[$i]=='-' && $sql_content[$i+1]=='-') {
                    // skip to next \n
                    for ($ii=$i;$ii<$chars;$ii++) {

                        if ($sql_content[$ii]=="\n") {
                            break;
                        } else {
                            $i++;
                        }

                    }
                } else {
                    if (!isset($query)) $query='';
                    $query.=$sql_content[$i];
                }
            }
        }

        foreach ($queries as $key => $val) {
            $query = trim($val);
            $query = str_replace('##_',$prefix,$query);
            if ($store_id!='') $query = str_replace('**_REPLACE_STORE_ID_**',$store_id,$query);

            // ok, now search vor OTHER INSERT INTO statements, and break them up
            if (substr($query,0,6)=='INSERT') {
                $check_qry = substr($query,7);

                if (strstr($check_qry,'INSERT')) {
                    $qry = explode('INSERT',$check_qry);
                    foreach ($qry as $k => $v) {
                        $queries[]='INSERT '.$v;
                    }
                    unset ($queries[$key]);
                } else {
                    $queries[$key]=$query;
                }
            } else {
                $queries[$key]=$query;
            }
        }
        return $queries;

    }


    /**
    * @param WizardPage $page
    */
    public function doUpdate(WizardPage $page)
    {
        global $store_handler, $language;

        $install_prefix = DB_PREFIX.'_';

        /** @var ADOConnection $idb */
        $idb = Wizard::getInstance()->getDatabaseObject();

        $sql_version = $idb->GetOne("SELECT VERSION() AS Version");
        if(version_compare($sql_version, '5.6') == -1)
        {
            die('Systemanforderung  mysql 5.6 minimum nicht erfüllt. <a href="https://xtcommerce.atlassian.net/wiki/spaces/MANUAL/pages/917573/Systemanforderungen" target="_blank">Handbuch</a>');
        }


        $update_sql = 'update.sql';
        $res = $this->_installSQL(Wizard::getInstance()->getDatabaseObject(), $update_sql, $install_prefix,'', '');

        foreach ($res as $key => $val) {
            $this->addControllerCommand(ControllerCommand::factory()
                ->setAction(function(ControllerCommand $command) use ($val, $idb) {
                    try{
                        $idb->Execute($val);
                    }catch (Exception $e){
                        /*'1060 - Duplicate column name','1062 - Duplicate entry', '1050- table alredy exists','1091 -  Can't DROP '%s'; check that column/key exists'*/
                        $allowed_error_codes = array(1062,1061,1060,1050,1091, 1054, 1091);
                        if (in_array($e->getCode(),$allowed_error_codes)) {
                            $command->setErrorMessage("<br/><span class='msg-skipped'>".TEXT_QUERY_SKIPPED."  " . $e->getMessage() . "</span>");
                        }else{
                            $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . ' | '.$e->getCode()."</span><br/>");
                            return false;
                        }
                    }

                    return true;
                })
                ->setDescription(TEXT_EXECUTING_QUERY.' '.$val)
                ->setAbortOnError(true)
            );
        }

        // fehlende keys in config_x prüfen und hinzufügen bzw aktualisieren

        // 4.2.00 5.0.08 date_added
        $this->addControllerCommand(ControllerCommand::factory()
            ->setAction(function(ControllerCommand $command) use ($idb, $store_handler) {

                $stores = $store_handler->getStores();
                foreach ($stores as $store)
                {
                    $table = DB_PREFIX.'_config_'.$store['id'];
                    try
                    {
                        $idb->Execute("UPDATE `".$table."` SET date_added = '1000-01-01 00:00:00' WHERE `date_added` IS NULL OR `date_added` < '1000-01-01 00:00:00';");
                    }
                    catch (Exception $e){
                        $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . "</span><br/>");
                    }
                }

                return true;
            })
            ->setDescription(constant('TEXT_UPDATING') . " DB _config_x update  date_added<br/>")
            ->setAbortOnError(true)
        );

        // 4.2.00 5.0.08 last_modified
        $this->addControllerCommand(ControllerCommand::factory()
            ->setAction(function(ControllerCommand $command) use ($idb, $store_handler) {

                $stores = $store_handler->getStores();
                foreach ($stores as $store)
                {
                    $table = DB_PREFIX.'_config_'.$store['id'];
                    try
                    {
                        $idb->Execute("UPDATE `".$table."` SET last_modified = NULL WHERE `last_modified` < '1000-01-01 00:00:00';");
                    }
                    catch (Exception $e){
                        $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . "</span><br/>");
                    }
                }

                return true;
            })
            ->setDescription(constant('TEXT_UPDATING') . " DB _config_x update  last_modified<br/>")
            ->setAbortOnError(true)
        );

        // 4.2.00 5.0.08 date_added 2
        $this->addControllerCommand(ControllerCommand::factory()
            ->setAction(function(ControllerCommand $command) use ($idb, $store_handler) {

                $stores = $store_handler->getStores();
                foreach ($stores as $store)
                {
                    $table = DB_PREFIX.'_config_'.$store['id'];
                    try
                    {
                        $idb->Execute("ALTER TABLE `".$table."` CHANGE COLUMN `date_added` `date_added` datetime NOT NULL DEFAULT current_timestamp, CHANGE COLUMN `last_modified` `last_modified` datetime NULL DEFAULT NULL ON UPDATE current_timestamp;");
                    }
                    catch (Exception $e){
                        $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . "</span><br/>");
                    }
                }

                return true;
            })
            ->setDescription(constant('TEXT_UPDATING') . " DB _config_x update  date_added 2<br/>")
            ->setAbortOnError(true)
        );

        // 5.1.x 6.0.1 _STORE_SHOPOWNER_REGISTERID
        $this->addControllerCommand(ControllerCommand::factory()
            ->setAction(function(ControllerCommand $command) use ($idb, $store_handler) {
                
                $new_key = '_STORE_SHOPOWNER_REGISTERID';

                $plg = new plugin();
                
                $default = '__NOT_SET__';
                if($plg->_FieldExists($new_key,DB_PREFIX.'_config_1'))
                {
                    $default = $idb->GetOne("SELECT config_value FROM " . DB_PREFIX . '_config_1' . " where config_key = ?", [$new_key]);
                }

                $stores = $store_handler->getStores();
                foreach ($stores as $store)
                {
                    $table = DB_PREFIX.'_config_'.$store['id'];
                    try
                    {
                        if(!$plg->_FieldExists($new_key, $table))
                        {
                            $value = $default != '__NOT_SET__' ? $default : '';
                            $idb->Execute("INSERT INTO `".$table."` (`config_key`, `config_value`, `group_id`, `sort_order`, `type`, `url`) VALUES (?, ?, 26, 11, NULL, NULL);", [$new_key, $default]);
                        }
                    }
                    catch (Exception $e){
                        $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . "</span><br/>");
                    }
                }

                return true;
            })
            ->setDescription(constant('TEXT_UPDATING') . " DB _config_x update  _STORE_SHOPOWNER_REGISTERID<br/>")
            ->setAbortOnError(true)
        );

        // 5.1.x 6.0.1 _STORE_SHOPOWNER_REGISTER_COURT
        $this->addControllerCommand(ControllerCommand::factory()
            ->setAction(function(ControllerCommand $command) use ($idb, $store_handler) {

                $new_key = '_STORE_SHOPOWNER_REGISTER_COURT';

                $plg = new plugin();

                $default = '__NOT_SET__';
                if($plg->_FieldExists($new_key,DB_PREFIX.'_config_1'))
                {
                    $default = $idb->GetOne("SELECT config_value FROM " . DB_PREFIX . '_config_1' . " where config_key = ?", [$new_key]);
                }

                $stores = $store_handler->getStores();
                foreach ($stores as $store)
                {
                    $table = DB_PREFIX.'_config_'.$store['id'];
                    try
                    {
                        if(!$plg->_FieldExists($new_key, $table))
                        {
                            $value = $default != '__NOT_SET__' ? $default : '';
                            $idb->Execute("INSERT INTO `".$table."` (`config_key`, `config_value`, `group_id`, `sort_order`, `type`, `url`) VALUES (?, ?, 26, 12, NULL, NULL);", [$new_key, $default]);
                        }
                    }
                    catch (Exception $e){
                        $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . "</span><br/>");
                    }
                }

                return true;
            })
            ->setDescription(constant('TEXT_UPDATING') . " DB _config_x update  _STORE_SHOPOWNER_REGISTER_COURT<br/>")
            ->setAbortOnError(true)
        );

        // 6.0.x 6.1.x _STORE_ACCOUNT_MIN_AGE
        // ist nur umsortieren fassen wir nicht an und wurde im nächtem updater gefixt


        // 6.5.x 6.6.x _STORE_SHOPOWNER_DUNS
        $this->addControllerCommand(ControllerCommand::factory()
            ->setAction(function(ControllerCommand $command) use ($idb, $store_handler) {

                $new_key = '_STORE_SHOPOWNER_DUNS';

                $plg = new plugin();

                $stores = $store_handler->getStores();
                foreach ($stores as $store)
                {
                    $table = DB_PREFIX.'_config_'.$store['id'];
                    try
                    {
                        if(!$plg->_FieldExists($new_key, $table))
                        {
                            $idb->Execute("INSERT INTO `".$table."` (`config_key`, `config_value`, `group_id`, `sort_order`, `type`, `url`) VALUES (?,'', 26, 13, NULL, NULL);", [$new_key]);
                        }
                    }
                    catch (Exception $e){
                        $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . "</span><br/>");
                    }
                }

                return true;
            })
            ->setDescription(constant('TEXT_UPDATING') . " DB _config_x update  _STORE_SHOPOWNER_DUNS<br/>")
            ->setAbortOnError(true)
        );

        // 6.5.x 6.6.x _STORE_SHOPOWNER_TAXID
        $this->addControllerCommand(ControllerCommand::factory()
            ->setAction(function(ControllerCommand $command) use ($idb, $store_handler) {

                $new_key = '_STORE_SHOPOWNER_TAXID';

                $plg = new plugin();

                $stores = $store_handler->getStores();
                foreach ($stores as $store)
                {
                    $table = DB_PREFIX.'_config_'.$store['id'];
                    try
                    {
                        if(!$plg->_FieldExists($new_key, $table))
                        {
                            $idb->Execute("INSERT INTO `".$table."` (`config_key`, `config_value`, `group_id`, `sort_order`, `type`, `url`) VALUES (?,'', 26, 14, NULL, NULL);", [$new_key]);
                        }
                    }
                    catch (Exception $e){
                        $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . "</span><br/>");
                    }
                }

                return true;
            })
            ->setDescription(constant('TEXT_UPDATING') . " DB _config_x update  _STORE_SHOPOWNER_TAXID<br/>")
            ->setAbortOnError(true)
        );

        // 6.5.x 6.6.x _STORE_SHOPOWNER_GLN
        $this->addControllerCommand(ControllerCommand::factory()
            ->setAction(function(ControllerCommand $command) use ($idb, $store_handler) {

                $new_key = '_STORE_SHOPOWNER_GLN';

                $plg = new plugin();

                $stores = $store_handler->getStores();
                foreach ($stores as $store)
                {
                    $table = DB_PREFIX.'_config_'.$store['id'];
                    try
                    {
                        if(!$plg->_FieldExists($new_key, $table))
                        {
                            $idb->Execute("INSERT INTO `".$table."` (`config_key`, `config_value`, `group_id`, `sort_order`, `type`, `url`) VALUES (?,'', 26, 14, NULL, NULL);", [$new_key]);
                        }
                    }
                    catch (Exception $e){
                        $command->setErrorMessage("<span class='msg-error'>".constant('TEXT_QUERY_FAILED')." " . $e->getMessage() . "</span><br/>");
                    }
                }

                return true;
            })
            ->setDescription(constant('TEXT_UPDATING') . " DB _config_x update  _STORE_SHOPOWNER_GLN<br/>")
            ->setAbortOnError(true)
        );

        $page->assignTemplateVar('processing',constant('TEXT_UPDATING'),true);
        $this->execAsyncAction($page);
    }

}

global $language;
if($language)
{
    $langFile = dirname(__FILE__) . DIRECTORY_SEPARATOR . "wizard_{$language->content_language}.yml";
    if (file_exists($langFile))
    {

        $contents = file_get_contents($langFile);
        $languageParts = explode("\n", $contents);

        foreach ($languageParts as $part)
        {
            $delimiterPos = strpos($part, "=", 0);

            if ($delimiterPos === false)
            {
                continue;
            }
            $systemParts = substr($part, 0, $delimiterPos);
            $value = substr($part, $delimiterPos + 1);

            list($plugin, $type, $key) = explode(".", $systemParts);
            if (!defined($key))
            {
                $value = str_replace(array("\r", "\n"), '', $value);
                define($key, $value);
            }
        }
    }
}
Wizard::getInstance()->registerScript(new Update65xTo66x());

    